/**
 * @file process 处理流程
 * @author zongyu(zongyu@baidu.com)
 *         wuqi03(wuqi03@baidu.com)
 */

var processCss = require('./processCss');
var format = require('./stringFormat');
var path = require('path');
var fs = require('fs');

var AMD_TPL = fs.readFileSync(path.resolve(__dirname, 'amd.tpl'));
var COMMONJS_TPL = fs.readFileSync(path.resolve(__dirname, 'commonjs.tpl'));

/**
 * 处理CSS Modules代码到AMD格式
 *
 * @param {string} content CSS的内容
 * @param {Object} options 配置项
 * @param {string} options.filePath 文件路径
 * @param {string} options.type 生成文件格式 amd | commonjs  默认amd
 * @param {Function=} options.processUrl 图片路径root
 * @param {Function=} options.processExportNamePrefix className格式处理
 * @param {Function=} options.processImportName 处理引用css路径
 * @param {Function=} callback 回调函数
 */
module.exports = function process(content, options, callback) {
    var exportNamePrefix = options.exportNamePrefix || exportNamePrefixFn;

    processCss(content, null, {
        root: '/',
        filePath: options.filePath,
        exportNamePrefix: exportNamePrefix(options.filePath)
    }, function (_, result) {
        var importData = getImportData(result.importItems, options);
        var type = options.type || 'amd';

        var today = new Date();
        var fileText = format(
            type === 'amd' ? AMD_TPL : COMMONJS_TPL,
            {
                requireList: getRequireList(importData.importList),
                source: getSource(result, importData, options),
                exportsData: getExportStr(result, importData, options),
                fileName: path.parse(options.filePath).name,
                time: today.getFullYear() + '-' + today.getMonth() + '-' + today.getDate()
            }
        );

        callback && callback(fileText);
    });
};

/**
 * 获取所有import的数据
 *
 * @inner
 * @param {Array} importItems import数据源
 * @param {Object} options 配置项
 * @return {Object}
 */
function getImportData(importItems, options) {
    var processImportName = options.processImportName || processImportNameFn;
    var importPathMap = {};
    var importList = {};
    var index = 0;
    importItems.forEach(function (item) {
        var url = processImportName(item.url);
        var fileName = path.basename(url, '.css');
        var importFlag = importPathMap[url];
        if (!importFlag) {
            importFlag = fileName + index++;
            importPathMap[url] = importFlag;
            importList[importFlag] = url;
        }
    });

    return {
        importPathMap: importPathMap,
        importList: importList
    };
}

/**
 * 获取requireList字符串
 *
 * @inner
 * @param {Object} importList import数据
 * @return {string}
 */
function getRequireList(importList) {
    var requireList = [];

    Object.keys(importList).forEach(function (key) {
        requireList.push('var ' + key + ' = require(\'' + importList[key] + '\');');
    });

    return requireList.join('\n    ');
}

/**
 * 处理source变量替换
 *
 * @inner
 * @param {Object} result processCss返回数据
 * @param {Object} importData 引用数据
 * @param {Object} options 配置参数
 * @return {string} source数据
 */
function getSource(result, importData, options) {
    var importItems = result.importItems;
    var urlItems = result.urlItems;
    var source = result.source;
    var importItemRegExpG = result.importItemRegExpG;
    var urlItemRegExpG = result.urlItemRegExpG;
    var processUrl = options.processUrl || processUrlFn;
    var processImportName = options.processImportName || processImportNameFn;
    var importPathMap = importData.importPathMap;

    source = '\''
        + source.replace(
            importItemRegExpG,
            function (_, num) {
                var importItemData = importItems[num];
                var url = processImportName(importItemData.url);
                var importFlag = importPathMap[url];
                return '\' + ' + importFlag + '.' + importItemData.export + ' + \'';
            }
        ).replace(
            urlItemRegExpG,
            function (_, num) {
                return processUrl(urlItems[num].url);
            }
        )
        + '\'';

    source = source.replace(/\n/g, '\\n').replace('/\t\r/', '');

    return source;
}

/**
 * 处理抛出数据
 *
 * @inner
 * @param {Object} result processCss返回数据
 * @param {Object} importData 引用数据
 * @param {Object} options 配置参数
 * @return {string} exportData数据
 */
function getExportStr(result, importData, options) {
    var importItems = result.importItems;
    var exportsData = result.exports;
    var importItemRegExpG = result.importItemRegExpG;
    var processImportName = options.processImportName || processImportNameFn;
    var importPathMap = importData.importPathMap;
    var exportsDataArr = [];

    Object.keys(exportsData).forEach(function (key) {
        var value = '\'' + exportsData[key].replace(/\"/g, '') + '\'';

        if (importItemRegExpG.test(value)) {
            value = value.replace(importItemRegExpG, function (_, num) {
                var importItemData = importItems[num];
                var url = processImportName(importItemData.url);
                var importFlag = importPathMap[url];
                return '\' + ' + importFlag + '.' + importItemData.export + ' + \'';
            });
        }

        exportsDataArr.push('\'' + key + '\': ' + value.replace(/\'\' \+| \+ \'\'/g, ''));
    });

    return '{\n        ' + exportsDataArr.join(',\n        ') + '\n    }';
}

/**
 * 处理css中url路径
 *
 * @inner
 * @param {string} url 源路径
 * @return {string}
 */
function processUrlFn(url) {
    return url || '';
}

/**
 * 生成class前缀
 *
 * @inner
 * @param {string} filePath 文件路径
 * @return {string}
 */
function exportNamePrefixFn(filePath) {
    return path.basename(filePath, '.css');
}

/**
 * 处理import css路径
 *
 * @inner
 * @param {string} url 源路径
 * @return {string}
 */
function processImportNameFn(url) {
    return url;
}
