/**
 * @file edp-build processor
 * @author wuqi03(wuqi03@baidu.com)
 */

var AbstractProcessor = require('edp-build/lib/processor/abstract');
var path = require('path');
var fs = require('fs');
var cssLoader = require('../index');
var util = require('util');

/**
 * 缓存文件目录
 *
 * @const
 */
var CACHE_FILE_PATH = path.resolve(process.cwd(), '.edpproj', 'edp-cache');

/**
 * 用于处理ES文件后缀
 *
 * @const
 * @type {RegExp}
 */
var res = /\.(css)$/i;

/**
 * cssloader处理器
 *
 * @constructor
 * @param {Object} options 配置项
 */
function CssLoaderProcessor(options) {
    if (!fs.existsSync(process.cwd(), '.edpproj')) {
        throw new Error('project is not an edp project, run `edp project init` first');
    }

    if (!fs.existsSync(CACHE_FILE_PATH)) {
        fs.mkdirSync(CACHE_FILE_PATH);
    }

    var ignore = path.resolve(process.cwd(), '.edpproj', '.gitignore');
    var content = fs.readFileSync(ignore, 'utf8');

    if (!/^edp-cache$/m.test(content)) {
        content += '\nedp-cache';
        fs.writeFileSync(ignore, content, 'utf8');
    }

    AbstractProcessor.call(this, options);
}

util.inherits(CssLoaderProcessor, AbstractProcessor);

CssLoaderProcessor.DEFAULT_OPTIONS = {
    name: 'CssLoaderProcessor',
    files: ['*.css']
};

/**
 * 构建处理
 *
 * @param {FileInfo} file 文件信息对象
 * @param {ProcessContext} processContext 构建环境对象
 * @param {Function} callback 处理完成回调函数
 */
CssLoaderProcessor.prototype.process = function (file, processContext, callback) {

    delete processContext.files[file.path];
    file.path += '.js';
    file.outputPath += '.js';
    file.fullPath += '.js';
    file.extname = 'js';
    processContext.addFile(file);

    var md5 = file.md5sum(0, 32);

    // 后边其它地方使用的时候，这个MD5可能不一样了
    file.set('md5sum', null);
    var code = file.data;

    var cachePath = path.resolve(CACHE_FILE_PATH, 'cssloader_' + file.path.replace(/\/|\\|\./g, '_') + '.cache');

    if (fs.existsSync(cachePath)) {
        var cacheContent = fs.readFileSync(cachePath, 'utf8');
        var backup = cacheContent.slice(0, 32);

        if (backup === md5) {
            file.setData(cacheContent.slice(32));
            callback();
            return;
        }
    }

    cssLoader(
        file.path.replace('.js', ''),
        {
            filePath: this.filePath,
            type: this.type,
            processUrl: this.processUrl,
            processExportNamePrefix: this.processExportNamePrefix,
            processImportName: this.processImportName
        },
        function (fileText) {
            file.setData(fileText);
            fs.writeFileSync(cachePath, md5 + fileText, 'utf8');
            callback();
        }
    );
};

module.exports = exports = CssLoaderProcessor;